@@ -83,15 +83,16 @@ STATIC int _drbd_md_sync_page_io(struct drbd_conf *mdev,
 	init_completion(&md_io.event);
 	md_io.error = 0;
 
-	if (rw == WRITE && !test_bit(MD_NO_BARRIER, &mdev->flags))
-		rw |= (1<<BIO_RW_BARRIER);
-#ifdef BIO_RW_UNPLUG
+	if ((rw & WRITE) && !test_bit(MD_NO_BARRIER, &mdev->flags))
+		rw |= (1UL << BIO_RW_BARRIER);
+#ifdef BIO_RW_SYNC
+	rw |= (1<<BIO_RW_SYNC);
+#else
 	/* see upstream commits
 	 * 213d9417fec62ef4c3675621b9364a667954d4dd,
-	 * 93dbb393503d53cd226e5e1f0088fe8f4dbaa2b8 */
+	 * 93dbb393503d53cd226e5e1f0088fe8f4dbaa2b8
+	 * later, the defines even became an enum ;-) */
 	rw |= (1<<BIO_RW_SYNCIO) | (1<<BIO_RW_UNPLUG);
-#else
-	rw |= (1<<BIO_RW_SYNC);
 #endif
 
  retry:
@@ -117,7 +118,7 @@ STATIC int _drbd_md_sync_page_io(struct drbd_conf *mdev,
 	/* check for unsupported barrier op.
 	 * would rather check on EOPNOTSUPP, but that is not reliable.
 	 * don't try again for ANY return value != 0 */
-	if (unlikely(bio_barrier(bio) && !ok)) {
+	if (unlikely(bio_rw_flagged(bio, BIO_RW_BARRIER) && !ok)) {
 		/* Try again with no barrier */
 		dev_warn(DEV, "Barriers not supported on meta data device - disabling\n");
 		set_bit(MD_NO_BARRIER, &mdev->flags);
@@ -161,7 +162,7 @@ int drbd_md_sync_page_io(struct drbd_conf *mdev, struct drbd_backing_dev *bdev,
 		iop = mdev->md_io_tmpp;
 
 		if (rw & WRITE) {
-			/* these are GFP_KERNEL pages, preallocated
+			/* these are GFP_KERNEL pages, pre-allocated
 			 * on device initialization */
 			void *p = page_address(mdev->md_io_page);
 			void *hp = page_address(mdev->md_io_tmpp);
@@ -256,7 +257,7 @@ void drbd_al_begin_io(struct drbd_conf *mdev, sector_t sector)
 	if (al_ext->lc_number != enr) {
 		/* drbd_al_write_transaction(mdev,al_ext,enr);
 		 * recurses into generic_make_request(), which
-		 * disalows recursion, bios being serialized on the
+		 * disallows recursion, bios being serialized on the
 		 * current->bio_tail list now.
 		 * we have to delegate updates to the activity log
 		 * to the worker thread. */
@@ -308,7 +309,7 @@ void drbd_al_complete_io(struct drbd_conf *mdev, sector_t sector)
 int
 w_al_write_transaction(struct drbd_conf *mdev, struct drbd_work *w, int unused)
 {
-	struct update_al_work *aw = (struct update_al_work *)w;
+	struct update_al_work *aw = container_of(w, struct update_al_work, w);
 	struct lc_element *updated = aw->al_ext;
 	const unsigned int new_enr = aw->enr;
 	const unsigned int evicted = aw->old_enr;
@@ -366,10 +367,8 @@ w_al_write_transaction(struct drbd_conf *mdev, struct drbd_work *w, int unused)
 	sector =  mdev->ldev->md.md_offset
 		+ mdev->ldev->md.al_offset + mdev->al_tr_pos;
 
-	if (!drbd_md_sync_page_io(mdev, mdev->ldev, sector, WRITE)) {
+	if (!drbd_md_sync_page_io(mdev, mdev->ldev, sector, WRITE))
 		drbd_chk_io_error(mdev, 1, TRUE);
-		drbd_io_error(mdev, TRUE);
-	}
 
 	if (++mdev->al_tr_pos >
 	    div_ceil(mdev->act_log->nr_elements, AL_EXTENTS_PT))
@@ -407,7 +406,7 @@ STATIC int drbd_al_read_tr(struct drbd_conf *mdev,
 	sector = bdev->md.md_offset + bdev->md.al_offset + index;
 
 	/* Dont process error normally,
-	 * as this is done before disk is atached! */
+	 * as this is done before disk is attached! */
 	if (!drbd_md_sync_page_io(mdev, bdev, sector, READ))
 		return -1;
 
@@ -561,13 +560,12 @@ STATIC BIO_ENDIO_TYPE atodb_endio BIO_ENDIO_ARGS(struct bio *bio, int error)
 	int uptodate = bio_flagged(bio, BIO_UPTODATE);
 
 	BIO_ENDIO_FN_START;
-	/* strange behaviour of some lower level drivers...
+	/* strange behavior of some lower level drivers...
 	 * fail the request by clearing the uptodate flag,
 	 * but do not return any error?! */
 	if (!error && !uptodate)
 		error = -EIO;
 
-	/* corresponding drbd_io_error is in drbd_al_to_on_disk_bm */
 	drbd_chk_io_error(mdev, error, TRUE);
 	if (error && wc->error == 0)
 		wc->error = error;
@@ -615,6 +613,7 @@ STATIC int atodb_prepare_unless_covered(struct drbd_conf *mdev,
 	}
 	/* bios[i] == NULL, the next not yet used slot */
 
+	/* GFP_KERNEL, we are not in the write-out path */
 	bio = bio_alloc(GFP_KERNEL, 1);
 	if (bio == NULL)
 		return -ENOMEM;
@@ -688,6 +687,7 @@ void drbd_al_to_on_disk_bm(struct drbd_conf *mdev)
 
 	nr_elements = mdev->act_log->nr_elements;
 
+	/* GFP_KERNEL, we are not in anyone's write-out path */
 	bios = kzalloc(sizeof(struct bio *) * nr_elements, GFP_KERNEL);
 	if (!bios)
 		goto submit_one_by_one;
@@ -708,7 +708,7 @@ void drbd_al_to_on_disk_bm(struct drbd_conf *mdev)
 			goto free_bios_submit_one_by_one;
 	}
 
-	/* unneccessary optimization? */
+	/* unnecessary optimization? */
 	lc_unlock(mdev->act_log);
 	wake_up(&mdev->al_wait);
 
@@ -740,8 +740,6 @@ void drbd_al_to_on_disk_bm(struct drbd_conf *mdev)
 
 	put_ldev(mdev);
 
-	if (wc.error)
-		drbd_io_error(mdev, TRUE);
 	kfree(bios);
 	return;
 
@@ -814,7 +812,7 @@ static int _try_lc_del(struct drbd_conf *mdev, struct lc_element *al_ext)
  * @mdev:	DRBD device.
  *
  * Removes all active extents form the activity log, waiting until
- * the reference count of each etry dropped to 0 first, of course.
+ * the reference count of each entry dropped to 0 first, of course.
  *
  * You need to lock mdev->act_log with lc_try_lock() / lc_unlock()
  */
@@ -837,11 +835,12 @@ void drbd_al_shrink(struct drbd_conf *mdev)
 
 STATIC int w_update_odbm(struct drbd_conf *mdev, struct drbd_work *w, int unused)
 {
-	struct update_odbm_work *udw = (struct update_odbm_work *)w;
+	struct update_odbm_work *udw = container_of(w, struct update_odbm_work, w);
 
 	if (!get_ldev(mdev)) {
 		if (DRBD_ratelimit(5*HZ, 5))
 			dev_warn(DEV, "Can not update on disk bitmap, local IO disabled.\n");
+		kfree(udw);
 		return 1;
 	}
 
@@ -987,7 +986,7 @@ void __drbd_set_in_sync(struct drbd_conf *mdev, sector_t sector, int size,
 
 	/* we clear it (in sync).
 	 * round up start sector, round down end sector.  we make sure we only
-	 * clear full, alligned, BM_BLOCK_SIZE (4K) blocks */
+	 * clear full, aligned, BM_BLOCK_SIZE (4K) blocks */
 	if (unlikely(esector < BM_SECT_PER_BIT-1))
 		return;
 	if (unlikely(esector == (nr_sectors-1)))
@@ -1012,7 +1011,7 @@ void __drbd_set_in_sync(struct drbd_conf *mdev, sector_t sector, int size,
 	if (count) {
 		/* we need the lock for drbd_try_clear_on_disk_bm */
 		if (jiffies - mdev->rs_mark_time > HZ*10) {
-			/* should be roling marks,
+			/* should be rolling marks,
 			 * but we estimate only anyways. */
 			if (mdev->rs_mark_left != drbd_bm_total_weight(mdev) &&
 			    mdev->state.conn != C_PAUSED_SYNC_T &&
@@ -1227,21 +1226,21 @@ int drbd_try_rs_begin_io(struct drbd_conf *mdev, sector_t sector)
 	spin_lock_irq(&mdev->al_lock);
 	if (mdev->resync_wenr != LC_FREE && mdev->resync_wenr != enr) {
 		/* in case you have very heavy scattered io, it may
-		 * stall the syncer undefined if we giveup the ref count
+		 * stall the syncer undefined if we give up the ref count
 		 * when we try again and requeue.
 		 *
 		 * if we don't give up the refcount, but the next time
 		 * we are scheduled this extent has been "synced" by new
 		 * application writes, we'd miss the lc_put on the
-		 * extent we keept the refcount on.
-		 * so we remembered which extent we had to try agin, and
+		 * extent we keep the refcount on.
+		 * so we remembered which extent we had to try again, and
 		 * if the next requested one is something else, we do
 		 * the lc_put here...
 		 * we also have to wake_up
 		 */
 
 		trace_drbd_resync(mdev, TRACE_LVL_ALL,
-				  "dropping %u, aparently got 'synced' by application io\n",
+				  "dropping %u, apparently got 'synced' by application io\n",
 				  mdev->resync_wenr);
 
 		e = lc_find(mdev->resync, mdev->resync_wenr);
@@ -1286,7 +1285,7 @@ int drbd_try_rs_begin_io(struct drbd_conf *mdev, sector_t sector)
 
 			goto try_again;
 		}
-		/* Do or do not. There is no try. -- Joda */
+		/* Do or do not. There is no try. -- Yoda */
 		e = lc_get(mdev->resync, enr);
 		bm_ext = e ? lc_entry(e, struct bm_extent, lce) : NULL;
 		if (!bm_ext) {
@@ -1348,7 +1347,8 @@ void drbd_rs_complete_io(struct drbd_conf *mdev, sector_t sector)
 	bm_ext = e ? lc_entry(e, struct bm_extent, lce) : NULL;
 	if (!bm_ext) {
 		spin_unlock_irqrestore(&mdev->al_lock, flags);
-		dev_err(DEV, "drbd_rs_complete_io() called, but extent not found\n");
+		if (DRBD_ratelimit(5*HZ, 5))
+			dev_err(DEV, "drbd_rs_complete_io() called, but extent not found\n");
 		return;
 	}
 
@@ -1476,7 +1476,7 @@ void drbd_rs_failed_io(struct drbd_conf *mdev, sector_t sector, int size)
 
 	/*
 	 * round up start sector, round down end sector.  we make sure we only
-	 * handle full, alligned, BM_BLOCK_SIZE (4K) blocks */
+	 * handle full, aligned, BM_BLOCK_SIZE (4K) blocks */
 	if (unlikely(esector < BM_SECT_PER_BIT-1))
 		return;
 	if (unlikely(esector == (nr_sectors-1)))
