@@ -32,7 +32,7 @@
 /* OPAQUE outside this file!
  * interface defined in drbd_int.h
 
- * convetion:
+ * convention:
  * function name drbd_bm_... => used elsewhere, "public".
  * function name      bm_... => internal to implementation, "private".
 
@@ -340,7 +340,7 @@ void drbd_bm_cleanup(struct drbd_conf *mdev)
 /*
  * since (b->bm_bits % BITS_PER_LONG) != 0,
  * this masks out the remaining bits.
- * Rerturns the number of bits cleared.
+ * Returns the number of bits cleared.
  */
 STATIC int bm_clear_surplus(struct drbd_bitmap *b)
 {
@@ -392,7 +392,7 @@ STATIC void bm_set_surplus(struct drbd_bitmap *b)
 	bm_unmap(p_addr);
 }
 
-STATIC unsigned long __bm_count_bits(struct drbd_bitmap *b, const int swap_endian, const enum km_type km)
+STATIC unsigned long __bm_count_bits(struct drbd_bitmap *b, const int swap_endian)
 {
 	unsigned long *p_addr, *bm, offset = 0;
 	unsigned long bits = 0;
@@ -400,7 +400,7 @@ STATIC unsigned long __bm_count_bits(struct drbd_bitmap *b, const int swap_endia
 
 	while (offset < b->bm_words) {
 		i = do_now = min_t(size_t, b->bm_words-offset, LWPP);
-		p_addr = __bm_map_paddr(b, offset, km);
+		p_addr = __bm_map_paddr(b, offset, KM_USER0);
 		bm = p_addr + MLPP(offset);
 		while (i--) {
 			catch_oob_access_start();
@@ -411,8 +411,9 @@ STATIC unsigned long __bm_count_bits(struct drbd_bitmap *b, const int swap_endia
 			bits += hweight_long(*bm++);
 			catch_oob_access_end();
 		}
-		__bm_unmap(p_addr, km);
+		__bm_unmap(p_addr, KM_USER0);
 		offset += do_now;
+		cond_resched();
 	}
 
 	return bits;
@@ -420,35 +421,12 @@ STATIC unsigned long __bm_count_bits(struct drbd_bitmap *b, const int swap_endia
 
 static unsigned long bm_count_bits(struct drbd_bitmap *b)
 {
-	return __bm_count_bits(b, 0, KM_IRQ1);
+	return __bm_count_bits(b, 0);
 }
 
 static unsigned long bm_count_bits_swap_endian(struct drbd_bitmap *b)
 {
-	return __bm_count_bits(b, 1, KM_USER0);
-}
-
-void _drbd_bm_recount_bits(struct drbd_conf *mdev, char *file, int line)
-{
-	struct drbd_bitmap *b = mdev->bitmap;
-	unsigned long flags, bits;
-
-	ERR_IF(!b) return;
-
-	/* IMO this should be inside drbd_bm_lock/unlock.
-	 * Unfortunately it is used outside of the locks.
-	 * And I'm not yet sure where we need to place the
-	 * lock/unlock correctly.
-	 */
-
-	spin_lock_irqsave(&b->bm_lock, flags);
-	bits = bm_count_bits(b);
-	if (bits != b->bm_set) {
-		dev_err(DEV, "bm_set was %lu, corrected to %lu. %s:%d\n",
-		    b->bm_set, bits, file, line);
-		b->bm_set = bits;
-	}
-	spin_unlock_irqrestore(&b->bm_lock, flags);
+	return __bm_count_bits(b, 1);
 }
 
 /* offset and len in long words.*/
@@ -485,11 +463,11 @@ STATIC void bm_memset(struct drbd_bitmap *b, size_t offset, int c, size_t len)
 
 /*
  * make sure the bitmap has enough room for the attached storage,
- * if neccessary, resize.
+ * if necessary, resize.
  * called whenever we may have changed the device size.
  * returns -ENOMEM if we could not allocate enough memory, 0 on success.
  * In case this is actually a resize, we copy the old bitmap into the new one.
- * Otherwise, the bitmap is initiallized to all bits set.
+ * Otherwise, the bitmap is initialized to all bits set.
  */
 int drbd_bm_resize(struct drbd_conf *mdev, sector_t capacity)
 {
@@ -593,13 +571,13 @@ int drbd_bm_resize(struct drbd_conf *mdev, sector_t capacity)
 	bm_unmap(p_addr);
 
 	(void)bm_clear_surplus(b);
-	if (!growing)
-		b->bm_set = bm_count_bits(b);
 
 	bm_end_info(mdev, __func__);
 	spin_unlock_irq(&b->bm_lock);
 	if (opages != npages)
 		bm_vk_free(opages, opages_vmalloced);
+	if (!growing)
+		b->bm_set = bm_count_bits(b);
 	dev_info(DEV, "resync bitmap: bits=%lu words=%lu\n", bits, words);
 
  out:
@@ -615,16 +593,12 @@ int drbd_bm_resize(struct drbd_conf *mdev, sector_t capacity)
  *
  * maybe bm_set should be atomic_t ?
  */
-unsigned long drbd_bm_total_weight(struct drbd_conf *mdev)
+static unsigned long _drbd_bm_total_weight(struct drbd_conf *mdev)
 {
 	struct drbd_bitmap *b = mdev->bitmap;
 	unsigned long s;
 	unsigned long flags;
 
-	/* if I don't have a disk, I don't know about out-of-sync status */
-	if (!get_ldev_if_state(mdev, D_NEGOTIATING))
-		return 0;
-
 	ERR_IF(!b) return 0;
 	ERR_IF(!b->bm_pages) return 0;
 
@@ -632,8 +606,17 @@ unsigned long drbd_bm_total_weight(struct drbd_conf *mdev)
 	s = b->bm_set;
 	spin_unlock_irqrestore(&b->bm_lock, flags);
 
-	put_ldev(mdev);
+	return s;
+}
 
+unsigned long drbd_bm_total_weight(struct drbd_conf *mdev)
+{
+	unsigned long s;
+	/* if I don't have a disk, I don't know about out-of-sync status */
+	if (!get_ldev_if_state(mdev, D_NEGOTIATING))
+		return 0;
+	s = _drbd_bm_total_weight(mdev);
+	put_ldev(mdev);
 	return s;
 }
 
@@ -778,7 +761,7 @@ static BIO_ENDIO_TYPE bm_async_io_complete BIO_ENDIO_ARGS(struct bio *bio, int e
 
 	BIO_ENDIO_FN_START;
 
-	/* strange behaviour of some lower level drivers...
+	/* strange behavior of some lower level drivers...
 	 * fail the request by clearing the uptodate flag,
 	 * but do not return any error?!
 	 * do we want to WARN() on this? */
@@ -899,7 +882,6 @@ STATIC int bm_rw(struct drbd_conf *mdev, int rw) __must_hold(local)
 	if (test_bit(BM_MD_IO_ERROR, &b->bm_flags)) {
 		dev_alert(DEV, "we had at least one MD IO ERROR during bitmap IO\n");
 		drbd_chk_io_error(mdev, 1, TRUE);
-		drbd_io_error(mdev, TRUE);
 		err = -EIO;
 	}
 
@@ -910,7 +892,7 @@ STATIC int bm_rw(struct drbd_conf *mdev, int rw) __must_hold(local)
 		/* flush bitmap to stable storage */
 		drbd_md_flush(mdev);
 	} else /* rw == READ */ {
-		/* just read, if neccessary adjust endianness */
+		/* just read, if necessary adjust endianness */
 		b->bm_set = bm_count_bits_swap_endian(b);
 		dev_info(DEV, "recounting of set bits took additional %lu jiffies\n",
 		     jiffies - now);
@@ -946,7 +928,7 @@ int drbd_bm_write(struct drbd_conf *mdev) __must_hold(local)
  * @mdev:	DRBD device.
  * @enr:	Extent number in the resync lru (happens to be sector offset)
  *
- * The BM_EXT_SIZE is on purpose exactle the amount of the bitmap covered
+ * The BM_EXT_SIZE is on purpose exactly the amount of the bitmap covered
  * by a single sector write. Therefore enr == sector offset from the
  * start of the bitmap.
  */
@@ -976,7 +958,6 @@ int drbd_bm_write_sect(struct drbd_conf *mdev, unsigned long enr) __must_hold(lo
 		    "(meta-disk sector %llus)\n",
 		    enr, (unsigned long long)on_disk_sector);
 		drbd_chk_io_error(mdev, 1, TRUE);
-		drbd_io_error(mdev, TRUE);
 		for (i = 0; i < AL_EXT_PER_BM_SECT; i++)
 			drbd_bm_ALe_set_all(mdev, enr*AL_EXT_PER_BM_SECT+i);
 	}
@@ -1076,13 +1057,13 @@ unsigned long _drbd_bm_find_next_zero(struct drbd_conf *mdev, unsigned long bm_f
 }
 
 /* returns number of bits actually changed.
- * for val != 0, we change 0 -> 1, return code positiv
+ * for val != 0, we change 0 -> 1, return code positive
  * for val == 0, we change 1 -> 0, return code negative
  * wants bitnr, not sector.
+ * expected to be called for only a few bits (e - s about BITS_PER_LONG).
  * Must hold bitmap lock already. */
-
 int __bm_change_bits_to(struct drbd_conf *mdev, const unsigned long s,
-	const unsigned long e, int val, const enum km_type km)
+	unsigned long e, int val, const enum km_type km)
 {
 	struct drbd_bitmap *b = mdev->bitmap;
 	unsigned long *p_addr = NULL;
@@ -1090,23 +1071,24 @@ int __bm_change_bits_to(struct drbd_conf *mdev, const unsigned long s,
 	unsigned long last_page_nr = -1UL;
 	int c = 0;
 
+	if (e >= b->bm_bits) {
+		dev_err(DEV, "ASSERT FAILED: bit_s=%lu bit_e=%lu bm_bits=%lu\n",
+				s, e, b->bm_bits);
+		e = b->bm_bits ? b->bm_bits -1 : 0;
+	}
 	for (bitnr = s; bitnr <= e; bitnr++) {
-		ERR_IF (bitnr >= b->bm_bits) {
-			dev_err(DEV, "bitnr=%lu bm_bits=%lu\n", bitnr, b->bm_bits);
-		} else {
-			unsigned long offset = bitnr>>LN2_BPL;
-			unsigned long page_nr = offset >> (PAGE_SHIFT - LN2_BPL + 3);
-			if (page_nr != last_page_nr) {
-				if (p_addr)
-					__bm_unmap(p_addr, km);
-				p_addr = __bm_map_paddr(b, offset, km);
-				last_page_nr = page_nr;
-			}
-			if (val)
-				c += (0 == __test_and_set_bit(bitnr & BPP_MASK, p_addr));
-			else
-				c -= (0 != __test_and_clear_bit(bitnr & BPP_MASK, p_addr));
+		unsigned long offset = bitnr>>LN2_BPL;
+		unsigned long page_nr = offset >> (PAGE_SHIFT - LN2_BPL + 3);
+		if (page_nr != last_page_nr) {
+			if (p_addr)
+				__bm_unmap(p_addr, km);
+			p_addr = __bm_map_paddr(b, offset, km);
+			last_page_nr = page_nr;
 		}
+		if (val)
+			c += (0 == __test_and_set_bit(bitnr & BPP_MASK, p_addr));
+		else
+			c -= (0 != __test_and_clear_bit(bitnr & BPP_MASK, p_addr));
 	}
 	if (p_addr)
 		__bm_unmap(p_addr, km);
@@ -1115,7 +1097,7 @@ int __bm_change_bits_to(struct drbd_conf *mdev, const unsigned long s,
 }
 
 /* returns number of bits actually changed.
- * for val != 0, we change 0 -> 1, return code positiv
+ * for val != 0, we change 0 -> 1, return code positive
  * for val == 0, we change 1 -> 0, return code negative
  * wants bitnr, not sector */
 int bm_change_bits_to(struct drbd_conf *mdev, const unsigned long s,
@@ -1150,12 +1132,82 @@ int drbd_bm_clear_bits(struct drbd_conf *mdev, const unsigned long s, const unsi
 	return -bm_change_bits_to(mdev, s, e, 0);
 }
 
-/* the same thing, but without taking the spin_lock_irqsave.
- * you must first drbd_bm_lock(). */
-int _drbd_bm_set_bits(struct drbd_conf *mdev, const unsigned long s, const unsigned long e)
+/* sets all bits in full words,
+ * from first_word up to, but not including, last_word */
+static inline void bm_set_full_words_within_one_page(struct drbd_bitmap *b,
+		int page_nr, int first_word, int last_word)
 {
-       /* WARN_ON(!bm_is_locked(b)); */
-       return __bm_change_bits_to(mdev, s, e, 1, KM_USER0);
+	int i;
+	int bits;
+	unsigned long *paddr = kmap_atomic(b->bm_pages[page_nr], KM_USER0);
+	for (i = first_word; i < last_word; i++) {
+		bits = hweight_long(paddr[i]);
+		paddr[i] = ~0UL;
+		b->bm_set += BITS_PER_LONG - bits;
+	}
+	kunmap_atomic(paddr, KM_USER0);
+}
+
+/* Same thing as drbd_bm_set_bits, but without taking the spin_lock_irqsave.
+ * You must first drbd_bm_lock().
+ * Can be called to set the whole bitmap in one go.
+ * Sets bits from s to e _inclusive_. */
+void _drbd_bm_set_bits(struct drbd_conf *mdev, const unsigned long s, const unsigned long e)
+{
+	/* First set_bit from the first bit (s)
+	 * up to the next long boundary (sl),
+	 * then assign full words up to the last long boundary (el),
+	 * then set_bit up to and including the last bit (e).
+	 *
+	 * Do not use memset, because we must account for changes,
+	 * so we need to loop over the words with hweight() anyways.
+	 */
+	unsigned long sl = ALIGN(s,BITS_PER_LONG);
+	unsigned long el = (e+1) & ~((unsigned long)BITS_PER_LONG-1);
+	int first_page;
+	int last_page;
+	int page_nr;
+	int first_word;
+	int last_word;
+
+	if (e - s <= 3*BITS_PER_LONG) {
+		/* don't bother; el and sl may even be wrong. */
+		__bm_change_bits_to(mdev, s, e, 1, KM_USER0);
+		return;
+	}
+
+	/* difference is large enough that we can trust sl and el */
+
+	/* bits filling the current long */
+	if (sl)
+		__bm_change_bits_to(mdev, s, sl-1, 1, KM_USER0);
+
+	first_page = sl >> (3 + PAGE_SHIFT);
+	last_page = el >> (3 + PAGE_SHIFT);
+
+	/* MLPP: modulo longs per page */
+	/* LWPP: long words per page */
+	first_word = MLPP(sl >> LN2_BPL);
+	last_word = LWPP;
+
+	/* first and full pages, unless first page == last page */
+	for (page_nr = first_page; page_nr < last_page; page_nr++) {
+		bm_set_full_words_within_one_page(mdev->bitmap, page_nr, first_word, last_word);
+		cond_resched();
+		first_word = 0;
+	}
+
+	/* last page (respectively only page, for first page == last page) */
+	last_word = MLPP(el >> LN2_BPL);
+	bm_set_full_words_within_one_page(mdev->bitmap, last_page, first_word, last_word);
+
+	/* possibly trailing bits.
+	 * example: (e & 63) == 63, el will be e+1.
+	 * if that even was the very last bit,
+	 * it would trigger an assert in __bm_change_bits_to()
+	 */
+	if (el <= e)
+		__bm_change_bits_to(mdev, el, e, 1, KM_USER0);
 }
 
 /* returns bit state
@@ -1194,7 +1246,7 @@ int drbd_bm_test_bit(struct drbd_conf *mdev, const unsigned long bitnr)
 	return i;
 }
 
-/* returns number of bits set */
+/* returns number of bits set in the range [s, e] */
 int drbd_bm_count_bits(struct drbd_conf *mdev, const unsigned long s, const unsigned long e)
 {
 	unsigned long flags;
@@ -1212,6 +1264,8 @@ int drbd_bm_count_bits(struct drbd_conf *mdev, const unsigned long s, const unsi
 	ERR_IF(!b->bm_pages) return 1;
 
 	spin_lock_irqsave(&b->bm_lock, flags);
+	if (bm_is_locked(b))
+		bm_print_lock_info(mdev);
 	for (bitnr = s; bitnr <= e; bitnr++) {
 		w = bitnr >> LN2_BPL;
 		if (page_nr != w >> (PAGE_SHIFT - LN2_BPL + 3)) {
