@@ -26,7 +26,6 @@
 #include <linux/module.h>
 #include <linux/drbd.h>
 #include <linux/ctype.h>
-#include <linux/marker.h>
 #include "drbd_int.h"
 #include "drbd_tracing.h"
 #include <linux/drbd_tag_magic.h>
@@ -166,7 +165,7 @@ static void probe_drbd_epoch(struct drbd_conf *mdev, struct drbd_epoch *epoch,
 
 	switch (ev) {
 	case EV_TRACE_ALLOC:
-		dev_info(DEV, "Allocat epoch %p/xxxx { } nr_epochs=%d\n", epoch, mdev->epochs);
+		dev_info(DEV, "Allocate epoch %p/xxxx { } nr_epochs=%d\n", epoch, mdev->epochs);
 		break;
 	case EV_TRACE_FREE:
 		dev_info(DEV, "Freeing epoch %p/%d { size=%d } nr_epochs=%d\n",
@@ -224,11 +223,11 @@ static void probe_drbd_actlog(struct drbd_conf *mdev, sector_t sector, char* msg
 }
 
 /**
- * drbd_print_buffer() - Hexdump arbitraty binary data into a buffer
+ * drbd_print_buffer() - Hexdump arbitrary binary data into a buffer
  * @prefix:	String is output at the beginning of each line output.
  * @flags:	Currently only defined flag: DBGPRINT_BUFFADDR; if set, each
  *		line starts with the virtual address of the line being
- *		outupt. If clear, each line starts with the offset from the
+ *		output. If clear, each line starts with the offset from the
  *		beginning of the buffer.
  * @size:	Indicates the size of each entry in the buffer. Supported
  * 		values are sizeof(char), sizeof(short) and sizeof(int)
@@ -387,12 +386,10 @@ static void probe_drbd_bio(struct drbd_conf *mdev, const char *pfx, struct bio *
 	const int rw = bio->bi_rw;
 	const int biorw      = (rw & (RW_MASK|RWA_MASK));
 	const int biobarrier = (rw & (1<<BIO_RW_BARRIER));
-
-	const int biosync =
-#ifdef BIO_RW_UNPLUG
-		(1<<BIO_RW_SYNCIO) | (1<<BIO_RW_UNPLUG);
+#ifdef BIO_RW_SYNC
+	const int biosync = (rw & (1<<BIO_RW_SYNC));
 #else
-		(1<<BIO_RW_SYNC);
+	const int biosync = (rw & ((1<<BIO_RW_SYNCIO) | (1<<BIO_RW_UNPLUG)));
 #endif
 
 	if (!is_mdev_trace(mdev, TRACE_LVL_ALWAYS))
@@ -463,6 +460,7 @@ static void probe_drbd_req(struct drbd_request *req, enum drbd_req_event what, c
 		[barrier_acked] = "barrier_acked",
 		[data_received] = "data_received",
 		[read_completed_with_error] = "read_completed_with_error",
+		[read_ahead_completed_with_error] = "reada_completed_with_error",
 		[write_completed_with_error] = "write_completed_with_error",
 		[completed_ok] = "completed_ok",
 	};
@@ -475,7 +473,7 @@ static void probe_drbd_req(struct drbd_request *req, enum drbd_req_event what, c
 	const unsigned long s = req->rq_state;
 
 	if (what != nothing) {
-		dev_info(DEV, "_req_mod(%p %c ,%s)\n", req, rw, rq_event_names[what]);
+		dev_info(DEV, "__req_mod(%p %c ,%s)\n", req, rw, rq_event_names[what]);
 	} else {
 		dev_info(DEV, "%s %p %c L%c%c%cN%c%c%c%c%c %u (%llus +%u) %s\n",
 			 msg, req, rw,
@@ -490,19 +488,19 @@ static void probe_drbd_req(struct drbd_request *req, enum drbd_req_event what, c
 			 req->epoch,
 			 (unsigned long long)req->sector,
 			 req->size,
-			 conns_to_name(mdev->state.conn));
+			 drbd_conn_str(mdev->state.conn));
 	}
 }
 
 
-#define peers_to_name roles_to_name
-#define pdsks_to_name disks_to_name
+#define drbd_peer_str drbd_role_str
+#define drbd_pdsk_str drbd_disk_str
 
 #define PSM(A)							\
 do {								\
 	if (mask.A) {						\
 		int i = snprintf(p, len, " " #A "( %s )",	\
-				A##s_to_name(val.A));		\
+				 drbd_##A##_str(val.A));	\
 		if (i >= len)					\
 			return op;				\
 		p += i;						\
