@@ -133,6 +133,7 @@ enum drbd_ret_codes {
 	ERR_VERIFY_RUNNING	= 149, /* DRBD 8.2 only */
 	ERR_DATA_NOT_CURRENT	= 150,
 	ERR_CONNECTED		= 151, /* DRBD 8.3 only */
+	ERR_PERM		= 152,
 
 	/* insert new ones above this line */
 	AFTER_LAST_ERR_CODE
@@ -277,10 +278,10 @@ enum drbd_state_ret_codes {
 };
 
 /* from drbd_strings.c */
-extern const char *conns_to_name(enum drbd_conns);
-extern const char *roles_to_name(enum drbd_role);
-extern const char *disks_to_name(enum drbd_disk_state);
-extern const char *set_st_err_name(enum drbd_state_ret_codes);
+extern const char *drbd_conn_str(enum drbd_conns);
+extern const char *drbd_role_str(enum drbd_role);
+extern const char *drbd_disk_str(enum drbd_disk_state);
+extern const char *drbd_set_st_err_str(enum drbd_state_ret_codes);
 
 #define SHARED_SECRET_MAX 64
 
