@@ -22,10 +22,10 @@
 
 extern const char *drbd_buildtag(void);
 
-#define REL_VERSION "8.3.2"
+#define REL_VERSION "8.3.7"
 #define API_VERSION 88
 #define PRO_VERSION_MIN 86
-#define PRO_VERSION_MAX 90
+#define PRO_VERSION_MAX 91
 
 #ifndef __CHECKER__   /* for a sparse run, we need all STATICs */
 #define DBG_ALL_SYMBOLS /* no static functs, improves quality of OOPS traces */
@@ -80,6 +80,9 @@ extern const char *drbd_buildtag(void);
 /* 2.6.29 and up no longer have swabb.h */
 //#define HAVE_LINUX_BYTEORDER_SWABB_H
 
+/* some vendor kernel have it backported. */
+#define HAVE_SET_CPUS_ALLOWED_PTR
+
 /* Some vendor kernels < 2.6.7 might define msleep in one or
  * another way .. */
 
@@ -94,5 +97,8 @@ extern const char *drbd_buildtag(void);
  */
 #define KERNEL_HAS_PROC_CREATE
 
+/* In 2.6.32 we finally fixed connector to pass netlink_skb_parms to the callback
+ */
+#define KERNEL_HAS_CN_SKB_PARMS
 
 #endif
